// $Id: gtk_display.cpp 31 2006-11-18 05:58:14Z jonmmorgan $ ($URL: http://xsword.svn.sourceforge.net/svnroot/xsword/trunk/gtk_display.cpp $)
// vim:ts=4 sw=4
//
//  This program is released under the GNU GPL version 2.  For more licensing
//  information, see the file LICENSING.

#include <versekey.h>
using sword::SWKey;
using sword::VerseKey;

#include "gtk_display.h"
#include "unicode.h"

static const char svn_id[] = "$Id: gtk_display.cpp 31 2006-11-18 05:58:14Z jonmmorgan $";

void GtkDispTags::initialise(void)	{
	table = gtk_text_tag_table_new();
	verseNumber = gtk_text_tag_new("verse_number");
	g_object_set(G_OBJECT(verseNumber), "foreground", "blue", NULL);
	currentVerse = gtk_text_tag_new("current_verse");
	g_object_set(G_OBJECT(currentVerse), "foreground", "green", NULL);
	gtk_text_tag_table_add(table, verseNumber);
	gtk_text_tag_table_add(table, currentVerse);
}

bool GtkDisp::tags_initialised;
GtkDispTags GtkDisp::tags;

GtkDisp::GtkDisp(GtkWidget *textView): textView(textView) {
	textBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textView));
}

char GtkEntryDisp::Display(SWModule &imodule) {
	char tmpBuf[255];
	GtkTextIter end;
	(const char *)imodule;	// snap to entry
	sprintf(tmpBuf, "[%s] ", imodule.KeyText());
	gtk_text_buffer_set_text(textBuffer, "", 0);
	gtk_text_buffer_get_end_iter(textBuffer, &end);
	gtk_text_buffer_insert_with_tags(textBuffer, &end, tmpBuf, -1, tags.verseNumber, NULL);
	text_buffer_insert_utf8_validate(textBuffer, &end, imodule.StripText(), -1);
	return 0;
}

char GtkChapDisp::Display(SWModule &imodule) {
	char tmpBuf[255]; 

	VerseKey *key = (VerseKey *)(SWKey *)imodule;
	key->Normalize();
	int curVerse = key->Verse();
	int curChapter = key->Chapter();
	int curBook = key->Book();
	char *lastmod = NULL;
	GtkTextIter end;
	GtkTextMark *mark;

	gtk_text_buffer_set_text(textBuffer, "", -1);
	for (key->Verse(1); (key->Book() == curBook && key->Chapter() == curChapter && !imodule.Error()); imodule++) {

		sprintf(tmpBuf, "%d. ", key->Verse());
		gtk_text_buffer_get_end_iter(textBuffer, &end);
		gtk_text_buffer_insert_with_tags(textBuffer, &end, tmpBuf, -1, tags.verseNumber, NULL);
		// needs check at this point to look for linked verses 
		if (lastmod == NULL || strcmp ((const char*)imodule, lastmod)) {
			gtk_text_buffer_get_end_iter(textBuffer, &end);
			if (key->Verse() == curVerse) { // highlight current verse 
				mark = gtk_text_buffer_create_mark(textBuffer, NULL, &end, TRUE);
				text_buffer_insert_with_tag_utf8_validate(textBuffer, &end,
						(&imodule)->StripText(), -1, tags.currentVerse);
			} // if ( key ....
			else {
				// Using StripText() to filter - use (const char *)imodule to see the raw output.
				text_buffer_insert_utf8_validate(textBuffer, &end, (&imodule)->StripText(), -1);
			} // else
			gtk_text_buffer_get_end_iter(textBuffer, &end);
			gtk_text_buffer_insert(textBuffer, &end, "  ", -1);
		} //if (strcmp .....
		if(lastmod != NULL)	{
			free(lastmod);
		}
		lastmod = g_strdup((const char *)imodule);
	} // for ....
	key->Verse(1);
	key->Chapter(1);
	key->Book(curBook);
	key->Chapter(curChapter);
	key->Verse(curVerse);
	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(textView), mark, 0, TRUE, 0.5, 0.5);
	return 0;
}
