// $Id: unicode.cpp 30 2006-11-18 04:54:34Z jonmmorgan $ ($URL: http://xsword.svn.sourceforge.net/svnroot/xsword/trunk/unicode.cpp $)
// vim:ts=4 sw=4
//
//  This program is released under the GNU GPL version 2.  For more licensing
//  information, see the file LICENSING.

#include <gtk/gtk.h>
#include "unicode.h"

static void toUtf8(const char *str, int len, const char **Utf8Str, int *Utf8Len);

static const char svn_id[] = "$Id: unicode.cpp 30 2006-11-18 04:54:34Z jonmmorgan $";

/* String manager routines, for Unicode support. */
char* GtkStringMgr::upperUTF8(char* text, const unsigned int maxlen) {
	char *retText;

	if (isUtf8(text)) {
		retText = g_utf8_strup(text, -1);
		if(maxlen != 0 && strlen(retText) > maxlen)	{
		   retText[maxlen] = '\0';	
		}
	} else {
		retText = upperLatin1(text);
	}
	return retText;
}

char* GtkStringMgr::upperLatin1(char* text) {
	char* retText = text;

	while (*text) {
		*text++ = toupper(*text);
	}
	return retText;
}

bool GtkStringMgr::supportsUnicode() const {
	return true;
}

const bool GtkStringMgr::isUtf8(const char *str) {
	return (bool)g_utf8_validate(str, -1, NULL);
}

void text_buffer_insert_utf8_validate(GtkTextBuffer *buff, GtkTextIter *iter,
		const char *str, int len)	{
	const char *Utf8Str;
	int Utf8Len;
	toUtf8(str, len, &Utf8Str, &Utf8Len);
	// This statement has no effect, but fixes a strange crash under Win32.
	// The problem does not occur when tags are used with the UTF8 validation.
	g_print("");
	gtk_text_buffer_insert(buff, iter, Utf8Str, Utf8Len);
}

/* XXX: Requires varargs to use more than one tag. */
void text_buffer_insert_with_tag_utf8_validate(GtkTextBuffer *buff,
		GtkTextIter *iter, const char *str, int len, GtkTextTag *tag)	{
	const char *Utf8Str;
	int Utf8Len;
	toUtf8(str, len, &Utf8Str, &Utf8Len);
	gtk_text_buffer_insert_with_tags(buff, iter, Utf8Str, Utf8Len, tag, NULL);
}

static void toUtf8(const char *str, int len, const char **Utf8Str, int *Utf8Len)	{
	if(g_utf8_validate(str, len, NULL))	{
		*Utf8Str = str;
		*Utf8Len = len;
	} else	{
		gsize bytes_read, bytes_written;
		*Utf8Str = g_locale_to_utf8(str, len, &bytes_read, &bytes_written, NULL);
		if(*Utf8Str == NULL)	{
			*Utf8Str = "Error converting to UTF-8";
			*Utf8Len = -1;
		} else	{
			*Utf8Len = (int)bytes_written;
		}
	}
}
